﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

//====================================================
// Описание работы классов и методов исходника на:
// https://www.interestprograms.ru
// Исходные коды программ и игр
// Автор исходных кодов Клочков Павел
//====================================================

namespace linesk
{
    // Класс графического элемента, состоит из квадратной ячейки 
    // и геометрической фигуры.
    // Геометрическая фигура может принимать разные формы,
    // при выборе игроком демонстрировать активность.
    public class GraphItem
    {
        // Предназначен для вызова Invalidate() родительского окна.
        // Используется в таймерах активности и исчезания.
        private readonly Control Parent = null;

        // Таймеры обеспечивают псевдомногопоточную работу приложения.
        private readonly Timer timerActive = new Timer();
        private readonly Timer timerVanish = new Timer();


        // Переменная величины пульсирования геометрической фигуры.
        private readonly int inflateSize = -4;


        // Изменения размера сторон геометрического элемента 
        // при пульсации и исчезновении.
        private int deltaSize = 0;


        /// <summary>
        /// Изменение вида геометрической фигуры во всех
        /// объектах класса GraphItem
        /// </summary>
        public static TypeGraphItem CurrentTypeGraphItem;


        /// <summary>
        /// Координаты графэлемента.
        /// </summary>
        public Rectangle CellCoordinate;


        /// <summary>
        /// Цвет для геометрической фигуры.
        /// </summary>
        public Color Color = Color.Black;

        /// <summary>
        /// Вид геометрической фигуры
        /// </summary>
        public enum TypeGraphItem { Rhombus, Ellipse, Rectangle };



        /// <summary>
        /// Заказной конструктор
        /// </summary>
        /// <param name="parent">ссылка на родительское окно</param>
        public GraphItem(Control parent)
        {
            Parent = parent;

            // ---- Инициализация таймеров ----

            // таймер активности геометрической фигуры
            timerActive.Interval = 40;
            timerActive.Tick += new EventHandler(TimerActive_Tick);
            timerActive.Enabled = false;

            // таймер визуального исчезания геометрической фигуры
            timerVanish.Interval = 40;
            timerVanish.Tick += new EventHandler(TimerVanish_Tick);
            timerVanish.Enabled = false;

            deltaSize = inflateSize;
        }


        #region Рисование графэлемента в родительском окне

        /// <summary>
        /// Декоративная кисть ячейки.
        /// </summary>
        /// <returns>готовая кисть</returns>
        Brush BrushCell()
        {
            // Подготовка координат для прорисовки ромба в центре ячейки.
            var point1 = new Point(CellCoordinate.Left + CellCoordinate.Width / 2, CellCoordinate.Top);
            var point2 = new Point(CellCoordinate.Left + CellCoordinate.Width, CellCoordinate.Top + CellCoordinate.Height / 2);
            var point3 = new Point(CellCoordinate.Left + CellCoordinate.Width / 2, CellCoordinate.Top + CellCoordinate.Height);
            var point4 = new Point(CellCoordinate.Left, CellCoordinate.Top + CellCoordinate.Height / 2);

            Point[] pt = { point1, point2, point3, point4 };

            // Рисуем объемность ячейки.
            GraphicsPath gp = new GraphicsPath();

            gp.AddRectangle(CellCoordinate);


            // Раскраска геометрической фигуры.
            PathGradientBrush pathBrush = new PathGradientBrush(gp)
            {
                SurroundColors = new Color[] { Color.FromArgb(150, 50, 50) },
                CenterPoint = new PointF(CellCoordinate.Left + CellCoordinate.Width / 2, CellCoordinate.Top + CellCoordinate.Height / 2),
                CenterColor = Color.Black
            };

            return pathBrush;
        }



        /// <summary>
        /// Рисование графики на поверхности окна приложения.
        /// </summary>
        /// <param name="g">Поверхность GDI+</param>
        public void Draw(Graphics g)
        {
            // Рисование ячейки, места для геометрической фигуры.
            g.FillRectangle(BrushCell(), CellCoordinate.X + 1,
                CellCoordinate.Y + 1, CellCoordinate.Width - 1, CellCoordinate.Height - 1);

            // При невидимой геометрической фигуре прорисовывается только квадрат ячейки.
            if (visible == false) return;

            // Вспомогательный квадрат для эффекта исчезания и эффекта 
            // активности в выбранном состоянии.
            Rectangle rectInflate = CellCoordinate;
            rectInflate.Inflate(deltaSize, deltaSize);



            // ---------- Формирование геометрической фигуры внутри ячейки -------------------

            // Подготовка координат для прорисовки ромба.
            // Другие фигуры строятся сразу на базе прямоугольника.
            var point1 = new Point(CellCoordinate.Left + CellCoordinate.Width / 2, rectInflate.Top);
            var point2 = new Point(rectInflate.Left + rectInflate.Width, CellCoordinate.Top + CellCoordinate.Height / 2);
            var point3 = new Point(CellCoordinate.Left + CellCoordinate.Width / 2, rectInflate.Top + rectInflate.Height);
            var point4 = new Point(rectInflate.Left, CellCoordinate.Top + CellCoordinate.Height / 2);

            Point[] pt = { point1, point2, point3, point4 };


            // Выбор вида графической фигуры.
            GraphicsPath gp = new GraphicsPath();
            switch (CurrentTypeGraphItem)
            {
                case TypeGraphItem.Ellipse:
                    gp.AddEllipse(rectInflate);
                    break;
                case TypeGraphItem.Rectangle:
                    gp.AddRectangle(rectInflate);
                    break;
                case TypeGraphItem.Rhombus:
                    gp.AddPolygon(pt);
                    break;
            }


            // При минимальном размере окна могут возникать,
            // при исчезании фигур, 
            // отрицательные значения ширины и высоты.
            // Для предотвращения сбоев предназначена эта строка.
            if (rectInflate.Width < 0 || rectInflate.Height < 0) return;


            // Раскраска геометрической фигуры.
            PathGradientBrush pathBrush = new PathGradientBrush(gp)
            {
                SurroundColors = new Color[] { this.Color },
                CenterPoint = new PointF(CellCoordinate.Left + CellCoordinate.Width / 2, CellCoordinate.Top + CellCoordinate.Height / 2),
                CenterColor = Color.White
            };



            // Рисование текущей геометрической фигуры.
            switch (CurrentTypeGraphItem)
            {
                case TypeGraphItem.Ellipse:
                    g.FillEllipse(pathBrush, rectInflate);
                    break;
                case TypeGraphItem.Rectangle:
                    g.FillRectangle(pathBrush, rectInflate);
                    break;
                case TypeGraphItem.Rhombus:
                    g.FillPolygon(pathBrush, pt);
                    break;
            }

        }

        #endregion

        #region Свойства графэлемента - видимости, активности, исчезания

        bool visible = false;
        /// <summary>
        /// Невидимым считается даже исчезающая геометрическая фигура.
        /// </summary>
        public bool Visible
        {
            get
            {
                if (vanish == true || visible == false)
                    return false;

                return true;
            }
            set
            {
                visible = value;
            }
        }



        bool active = false;
        /// <summary>
        /// Свойство активности геометрической фигуры.
        /// </summary>
        public bool Active
        {
            get
            {
                return active;
            }
            set
            {
                active = value;

                if (active == true)
                {
                    timerActive.Enabled = true;
                }
                else
                {
                    timerActive.Enabled = false;
                    deltaSize = inflateSize;
                }
            }
        }



        bool vanish = false;
        /// <summary>
        /// Свойство плавного исчезания геометрической фигуры.
        /// </summary>
        public bool Vanish
        {
            get
            {
                return vanish;
            }
            set
            {
                vanish = value;

                if (vanish == true)
                {
                    timerVanish.Enabled = true;
                }
            }
        }

        #endregion

        #region Таймеры графэлемента - таймер активности и таймер исчезания


        int k = 1;
        /// <summary>
        /// Таймер активности периодически увеличивает и уменьшает геометрическую фигуру,
        /// создавая эффект активности.
        /// </summary>
        void TimerActive_Tick(object sender, EventArgs e)
        {
            deltaSize += 1 * k;


            // Цикл уменьшения и увеличения размера
            // геометрической фигуры при пульсировании.
            if (deltaSize >= -2)
            {
                k = -1;
            }
            else if (deltaSize <= -7)
            {
                k = 1;
            }

            // Для экономии процессорных ресурсов
            // обновляем только прямоугольник 
            // графического элемента.
            Parent.Invalidate(CellCoordinate);
        }




        /// <summary>
        /// Таймер исчезания сначала графэлемент уменьшается,
        /// а потом исчезает.
        /// </summary>
        void TimerVanish_Tick(object sender, EventArgs e)
        {

            deltaSize += -2;

            if (deltaSize < -14)
            {
                timerVanish.Enabled = false;
                visible = false;
                vanish = false;

                // После того как геометрическая фигура стала невидимой,
                // возвращаем ей нормальные размеры.
                deltaSize = inflateSize;
            }

            // Для экономии процессорных ресурсов
            // обновляем только прямоугольник 
            // графического элемента.
            Parent.Invalidate(CellCoordinate);
        }
        #endregion 

    }
}
